/**
 * DetailiSpindel.cpp is part of bmsapp.
 *
 * bmsapp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * bmsapp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "DetailiSpindel.h"
#include "../ui/ui_DetailiSpindel.h"
#include "global.h"
#include "Utils.h"
#include "MainWindow.h"


/*
 * Results are available via MySQL and websockets. Because we initialize using
 * MySQL we only use that for the results and up to date status.
 * Commands are send via websockets only.
 */

DetailiSpindel::DetailiSpindel(int id, QWidget *parent) : QDialog(parent), ui(new Ui::DetailiSpindel)
{
    QSqlQuery query;

    qDebug() << "DetailiSpindel record:" << id;
    ui->setupUi(this);
    this->recno = id;
    setWindowFlags(Qt::Window | Qt::WindowTitleHint | Qt::CustomizeWindowHint);
    setWindowTitle(tr("BMSapp - Details iSpindel"));

    ui->thermoMeter->setMaximum(40.0);
    ui->thermoMeter->setNominal(20.0);
    ui->thermoMeter->setCritical(25.0);
    ui->thermoMeter->setSuffix(QString("°C"));

    ui->modeEdit->addItem("OFF");
    ui->modeEdit->addItem("ON");

    ui->codePick->addItem("Free - Dummy");	// Will be replaced later
    query.exec("SELECT code,name FROM products WHERE stage='1' OR stage='2' OR stage='3' OR stage='4' OR stage='5' OR stage='6' OR stage='7' ORDER BY code");
    while (query.next()) {
	ui->codePick->addItem(query.value("code").toString()+" - "+query.value("name").toString());
    }

    connect(ui->codePick, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DetailiSpindel::code_changed);
    connect(ui->modeEdit, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DetailiSpindel::mode_changed);
    connect(parent, SIGNAL(updateiSpindel(QString)), this, SLOT(refreshiSpindel(QString)));
    emit refreshTable();
}


void DetailiSpindel::refreshTable()
{
    QSqlQuery query;

    qDebug() << "refreshTable ispindel rec:" << this->recno;

    query.prepare("SELECT * FROM mon_ispindels WHERE record = :recno");
    query.bindValue(":recno", this->recno);
    query.exec();
    if (query.next()) {

	const QSignalBlocker blocker1(ui->codePick);
	const QSignalBlocker blocker2(ui->modeEdit);

	_node = query.value("node").toString();
	_alias = query.value("alias").toString();
	_uuid = query.value("uuid").toString();
	_beercode = query.value("beercode").toString();
	_beername = query.value("beername").toString();

	ui->uuidEdit->setText(_uuid);
	ui->systemEdit->setText(_node+"/"+_alias);
	ui->codePick->setItemText(0, _alias.toUpper()+" - "+_alias);

	if (query.value("online").toInt()) {
	    ui->statusEdit->setText(tr("Online"));
            ui->codeEdit->setText(_beercode+" - "+_beername);
	    ui->modeEdit->show();
	    if (query.value("mode").toString() == "OFF") {
	    	ui->powerLED->setChecked(false);
		ui->modeEdit->setCurrentIndex(0);
		ui->codePick->show();
	    } else {
	    	ui->powerLED->setChecked(true);
		ui->modeEdit->setCurrentIndex(1);
		ui->codePick->hide();
	    }
	    ui->alarmLED->setChecked((query.value("alarm").toInt() != 0) ? true:false);

	    ui->voltEdit->setText(QString("%1").arg(query.value("battery").toDouble(), 4, 'f', 3, '0'));
	    ui->tiltEdit->setText(QString("%1").arg(query.value("angle").toDouble(), 6, 'f', 5, '0'));
	    ui->platoEdit->setText(QString("%1").arg(query.value("gravity").toDouble(), 4, 'f', 3, '0'));
            double sg = Utils::plato_to_sg(query.value("gravity").toDouble());
	    ui->sgEdit->setText(QString("%1").arg(sg, 5, 'f', 4, '0'));

	    if (query.value("gravity").toDouble()) {
		double og = Utils::plato_to_sg(query.value("og_gravity").toDouble());
		double svg = Utils::calc_svg(og, sg);

		ui->ogVal->setText(QString("%1").arg(og, 5, 'f', 4, '0'));
		ui->ogVal2->setText(QString("%1°P").arg(query.value("og_gravity").toDouble(), 4, 'f', 3, '0'));
		ui->sgVal->setText(QString("%1").arg(sg, 5, 'f', 4, '0'));
		ui->sgVal2->setText(QString("%1°P").arg(query.value("gravity").toDouble(), 4, 'f', 3, '0'));
		ui->svgVal->setText(QString("%1%").arg(svg, 2, 'f', 1, '0'));
		ui->svgBar->setValue(svg);
		ui->abvVal->setText(QString("%1%").arg(Utils::abvol(og, sg), 3, 'f', 2, '0'));
	    } else {
		ui->ogVal->setText("");
		ui->ogVal2->setText("");
		ui->sgVal->setText("");
		ui->sgVal2->setText("");
		ui->svgVal->setText("");
		ui->abvVal->setText("");
		ui->batVal->setText("");
	    }

	    double batt = query.value("battery").toDouble() - 3.064;	// 0% voltage
	    if (batt < 0)
		batt = 0;
	    batt = round(batt / 1.17875 * 100.0);	// 100% range
	    if (batt > 100)
		batt = 100;
	    ui->batVal->setText(QString("%1%").arg(batt, 2, 'f', 1, '0'));

	    ui->thermoBox->show();
	    ui->thermoMeter->setValue(query.value("temperature").toDouble());

	} else {
	    /* Offline */
	    ui->statusEdit->setText(tr("Offline"));
	    ui->powerLED->setChecked(false);
	    ui->alarmLED->setChecked(true);
	    ui->codePick->hide();
	    ui->modeEdit->hide();
	    ui->thermoBox->hide();
	    ui->logButton->hide();
	}
    }

}


DetailiSpindel::~DetailiSpindel()
{
    qDebug() << "DetailiSpindel done";
    delete ui;
    emit entry_changed();
}


/*
 * Receive signals destined for all co2meters.
 * Check if the signal is for us.
 */
void DetailiSpindel::refreshiSpindel(QString data)
{
    if (_node+"/"+_alias == data) {
	emit refreshTable();
    }
}


void DetailiSpindel::on_quitButton_clicked()
{
    this->close();
    this->setResult(1);
}


void DetailiSpindel::mode_changed(int val)
{
    QStringList mode ({ "OFF", "ON" });
    QString msg = QString("{\"device\":\"ispindels\",\"node\":\"" + _node + "\",\"unit\":\"" + _alias + "\",\"mode\":\"" + mode[val] + "\"}");
    qDebug() << "mode_changed" << val << msg;
    webSocket->sendTextMessage(msg);
}


void DetailiSpindel::code_changed(int val)
{
    QJsonParseError parseError;
    QSqlQuery query;

    QString msg = QString("{\"device\":\"ispindels\",\"node\":\"" + _node + "\",\"unit\":\"" + _alias + "\",");
    if (val == 0) {
	msg.append(QString("\"beeruuid\":\"") + _uuid + "\",");
	msg.append(QString("\"beercode\":\"") + _alias.toUpper() + "\",");
	msg.append(QString("\"beername\":\"") + _alias + "\"}");
    } else {
	query.exec("SELECT code,name,uuid,stage,json_yeasts FROM products WHERE stage='1' OR stage='2' OR stage='3' OR stage='4' OR stage='5' OR stage='6' OR stage='7' ORDER BY code");
	for (int i = 0; i < val; i++) {
            query.next();
        }
	msg.append(QString("\"beeruuid\":\"") + query.value("uuid").toString() + "\",");
	msg.append(QString("\"beercode\":\"") + query.value("code").toString() + "\",");
	msg.append(QString("\"beername\":\"") + query.value("name").toString() + "\"}");
    }

    qDebug() << "code_changed" << val << msg;
    webSocket->sendTextMessage(msg);
}

